function [subj_idx_module,COV_mean,COV_mean_module,IC_groups_idx] = Subgroup_identification_auto(Info_path,varargin)
% [subj_idx_module,COV_mean,COV_mean_module,IC_groups_idx] = ...
%              Subgroup_identification_auto(Info_path,'N_clulster_SCV', 0,'cov_t', 0.2,'q_t', 0.6, 'method', 1, 'N_subgroups', 3, 'type', 'clustering','N_max',10,''type','modularization'')

% For more details, please refer to https://mlsp.umbc.edu/IVA-CS.html

%%% This function automatically detect subgroups of subjects from
%%% IVA results in three steps:
%%%     1. Roughly devide the SCVs into three subspaces: 
%%%             first group of N1 SCVs with all components highly correlated, 
%%%             second group of N2 SCVs with partial components highly correlated
%%%             third group of N3 SCVs with all components slightly correlated)
%%%     2. Cluster the SCVs in the second group into N_cluster_SCV clusters
%%%     3. Identify subgroups of subjects for each cluster

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%---### INPUT ####---%
%%% Info_path: Directory of the info file of IVA (*_ica_parameter_info.mat) %%%%%%

%%% ---Optional input:
%%%    N_clulster_SCV: Number of SCV clusters in step 2
%%%    N_subgroups: Number of subgroups to be identified from group-specific SCVs. 
%%%                 If not provided, will be estimated in function identify_subgroups.m by calling get_cluster
%%%    cov_t: Threshold of correlation value used to compute the commonality q (as defined in the reference paper)
%%%    q_t: Threshold used to dintinguish the second and third groups
%%%    method: The method used to determine the number of SCV clusters in step 2.
%%%            '0' means user have to provide N_clulster_SCV
%%%    type: The method used to determine the number of SCV clusters in step 2
%%%    N_max: The maximum number of clusters within a matrix


%---### OUTPUT ####---%
%%% subj_idx_module: a cell returns the subject indices in each subgroup
%%%                  for each SCV cluster. Eg., subj_idx_module{i}{j}
%%%                  stores the subject indices in subgroup j for the ith
%%%                  SCV cluster
%%% COV_mean: a three way tensor returns the mean covariance matrix for
%%%           each SCV scluter
%%% COV_mean_module: a three way tensor returns the mean covariance matrix for
%%%           each SCV scluter after reagranging acoording to subgroups
%%% IC_groups_idx: a cell returns the IC (SCV) indices in each SCV cluster
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% ----------- by Q. Long ------ Oct 2020 --------------------%
% Q. Long, S. Bhinge, V.D. Calhoun, and T. Adal{\i}. (2020) ``Independent Vector Analysis for Common Subspace Analysis: 
% Application to Multi-subject fMRI Data Yields Meaningful Subgroups of Schizophrenia." NeuroImage, 116872.
% ----------- Updated by H. Yang ------ Sep 2023 --------------------%
% H. Yang, T. Vu, Q. Long, V. Calhoun, & T. Adal{\i}. (2023). ``Identification of Homogeneous Subgroups from 
% Resting-State fMRI Data." Sensors, 23(6), 3264.

% H. Yang, F. Ghayem, B. Gabrielson, M. A. B. S. Akhonda, V. D. Calhoun,& T. Adal{\i}. 
% ``Constrained Independent Component Analysis Based on Entropy Bound Minimization for 
% Subgroup Identification from Multi-subject fMRI Data," ICASSP, IEEE, 2023. 

% H. Yang, M. A. B. S. Akhonda, F. Ghayem,Q. Long, V. D. Calhoun,& T. Adal{\i}. 
% ``Independent vector analysis based subgroup identification from multisubject fMRI data," ICASSP, IEEE, 2022. 

% Copyright (C) 2023 MLSP Lab
% 
% This program is free software: you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation, either version 3 of the License, or
% (at your option) any later version.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
% GNU General Public License for more details. 
% Please make sure you include the references in any related work.
% 
% A copy of the GNU General Public License can be found at 
% <https://mlsp.umbc.edu/resources.html>.
% If not, see <https://www.gnu.org/licenses/>.
% 
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


Params=struct( ...
    'N_clulster_SCV', 0,... 
    'cov_t', 0.2, ... 
    'q_t', 0.6, ...  % Hanlu: May need to change values of q_t and cov_t according to different data's COV distribution 
    'method', 1, ... 
    'N_subgroups', 0, ... 
    'N_max',10, ... % Hanlu: Added N_max to have a flexible maximum number of clusters, N_max should be >5
    'type', 'clustering' ...
    );% Hanlu: type: modularization or clustering or gershgorin
Params=getopt(Params,varargin{:});
N_cluster_SCV = 0;
if isfield(Params,'N_cluster_SCV')
    N_cluster_SCV = Params.N_cluster_SCV;
end
N_subgroups = 0;
if isfield(Params,'N_subgroups')
    N_subgroups = Params.N_subgroups;
end
cov_t = 0.2; 
if isfield(Params,'cov_t')
    cov_t = Params.cov_t;
end
q_t = 0.6; 
if isfield(Params,'q_t')
    q_t = Params.q_t;
end
method = 1; 
if isfield(Params,'method')
    method = Params.method;
end
type = 'clustering';
if isfield(Params,'type')
    type = Params.type;
end
if isfield(Params,'N_max')
    N_max = Params.N_max;
end

%% Loading data and compute the correlation matrices of SCVs
load(Info_path)
files = dir([sesInfo.outputDir,'/',sesInfo.calibrate_components_mat_file,'*.mat']);
N_ic = sesInfo.numComp;
V = length(sesInfo.mask_ind);
K = length(files);
IC = zeros(N_ic,V,K); 
for k = 1:K
    load([sesInfo.outputDir,'/',files(k).name])
    IC(:,:,k) = zscore(ic')';
    clear tc ic
end
COV_mat = zeros(K,K,N_ic);
COV_vec = zeros(K*(K-1)/2,N_ic);
q = zeros(N_ic,1);
for n = 1:size(IC,1)
    COV_mat(:,:,n) = cov(squeeze(IC(n,:,:)));
    COV_vec(:,n) = A_Mat2Vec(squeeze(abs(COV_mat(:,:,n))),1);
    q(n) = length(find(COV_vec(:,n)>cov_t))/length(COV_vec(:,n));
end

%plot_COV(Cov_vec, q)% A helping function that visualizes the corr values of SCVs

%% Clustering the SCVs in the second group --- the group-spesific SCVs
% IDXall stores the indices of each cluster
N_SCV_1st = sum(q==1);
COV_vec_2nd = COV_vec(:,q<1&q>=q_t);
data = COV_vec_2nd';
COV_mat_2nd = COV_mat(:,:,q<1&q>=q_t);


%%%%%%%%%%%%%%%%%%%%%% Determine the number of clusters %%%%%%%%%%%%%%%%%%%%%%%%
%if N_cluster_SCV is not provided, use one of the three methods to compute/estimate it
if method==0 || method==1 || method==2 
    
    if method == 0 && N_cluster_SCV == 0 
        error('Error: Please provide the number of SCV clusters, or select a method for its estimation')
        
    elseif method == 1
%================ Method 1 ==============%
        N_cluster_SCV = floor(size(COV_vec_2nd,2)/3);
        
    elseif method == 2
%================ Method 2: need GIFT to be installed. Refer to function ===============%
%====== "icatb_optimal_clusters" for the selection of knee point detection method ======%
        N_run = 100;
        K_max = min( size(data,1)-1,N_max); % Hanlu: Changed from K_max = 10 to make sure #cluster <= #rows of data
        
        if K_max < 2
            error ('The rows of data should be greater than 2, change "cov_t" or "q_t" or "N_max" to avoid this error')% Hanlu: add error message to avoid clustering errors
        end 
        
        N_cluster_SCV = get_clusters(data,K_max,N_run,2);% read 'icatb_optimal_clusters.m', N_max should be >5
    end
    
    
    DEV = std(data, [], 2);
    [~, imax, ~, ~] = icatb_extrema(DEV);
    Cp = data(imax,:);
    clear IDXall SUMDall
    for run = 1:10
        [~, Cent, ~, ~] = icatb_kmeans(Cp, N_cluster_SCV, 'distance', 'Correlation',...
            'Replicates', 1, 'Display', 'iter', 'MaxIter', 150, 'empty', 'drop');
        [IDSCVall(:,run), ~, SUMSCVall(:,run), ~] = icatb_kmeans(data, N_cluster_SCV, 'distance', 'Correlation',...
            'Replicates', 1, 'Display', 'iter', 'MaxIter', 150, 'empty', 'drop','Start',Cent);
    end
    [~,best_run] = min(mean(SUMSCVall,1));
    IDSCVall = IDSCVall(:,best_run);
    
elseif method==3
%======================== Method 3: make use of modularity =============================%
%================ code from BCT (https://sites.google.com/site/bctnet/) ================%
    N_run = 100;
    K_max = min( size(data,1)-1,N_max); % Hanlu: Changed from K_max = 10 to make sure #cluster < #rows of data
    
    if K_max < 2
        error ('The rows of data should be greater than 2, change "cov_t" or "q_t" or "N_max" to avoid this error')% Hanlu: add error message to avoid clustering errors
    end 
    
%     [N_cluster_SCV,IDSCVall] = get_clusters(data,K_max,N_run,3);
        [N_cluster_SCV,IDSCVall] = get_clusters(data,K_max,N_run,3,'COV_mat',COV_mat_2nd); % Hanlu: update the input of get_clusters 


end

%% Identify subgroups from the mean COV of each SCV cluster 

%============= Type a: Use modularization as in the following paper ===========%
% Q. Long, S. Bhinge, V.D. Calhoun, and T. Adal{\i}. (2020) ``Independent Vector Analysis for Common Subspace Analysis: 
% Application to Multi-subject fMRI Data Yields Meaningful Subgroups of Schizophrenia." NeuroImage, 116872.

%========= Type b: Clustering by using the rows of the mean COV in each SCV cluster ========%
%%%%%%%%%%% Need GIFT to be installed. Uses function "icatb_optimal_clusters" for %%%%%%%%%%%%
%%%%%%%%%%% the selection of knee point detection method   %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%============= Type c: Use gershgorin as in the following paper ===========%
% H. Yang, M.A.B.S. Akhonda, F. Ghayem, Q. Long, V.D. Calhoun, and T. Adal{\i} (2022)
% `` Independent vector analysis based subgroup identification from multisubject fMRI data." ICASSP 2022(pp. 1471-1475).


[subj_idx_module,COV_mean,COV_mean_module,IC_groups_idx] = identify_subgroups(COV_mat_2nd,IDSCVall,K,N_cluster_SCV,N_subgroups,type);
for i = 1:N_cluster_SCV
    IC_groups_idx{i} = IC_groups_idx{i} + N_SCV_1st;
end


% plot the mean COV before and after identifying subgroups
for i = 1:N_cluster_SCV
    A_plot_Corr(COV_mean(:,:,i))
    A_plot_Corr(COV_mean_module(:,:,i))
end

end